﻿namespace HeuristicLab.BenchmarkSuite {

  using HeuristicLab.Common;
  using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

  [StorableClass]
  public class Example : DeepCloneable {
    private static readonly long[] emptyIntegers = new long[0];
    private static readonly double[] emptyFloats = new double[0];
    private static readonly bool[] emptyBooleans = new bool[0];
    private static readonly char[] emptyChars = new char[0];
    private static readonly string[] emptyStrings = new string[0];


    public Example() {
      InputInt = emptyIntegers;
      InputFloat = emptyFloats;
      InputBoolean = emptyBooleans;
      InputChar = emptyChars;
      InputString = emptyStrings;

      OutputInt = emptyIntegers;
      OutputFloat = emptyFloats;
      OutputBoolean = emptyBooleans;
      OutputChar = emptyChars;
      OutputString = emptyStrings;
    }

    public Example(Example origin, Cloner cloner) : base(origin, cloner) {
      InputArgs = (string[])origin.InputArgs.Clone();
      OutputArgs = (string[])origin.OutputArgs.Clone();

      InputBoolean = (bool[])origin.InputBoolean.Clone();
      InputInt = (long[])origin.InputInt.Clone();
      InputFloat = (double[])origin.InputFloat.Clone();
      InputChar = (char[])origin.InputChar.Clone();
      InputString = (string[])origin.InputString.Clone();

      OutputBoolean = (bool[])origin.OutputBoolean.Clone();
      OutputInt = (long[])origin.OutputInt.Clone();
      OutputFloat = (double[])origin.OutputFloat.Clone();
      OutputChar = (char[])origin.OutputChar.Clone();
      OutputString = (string[])origin.OutputString.Clone();
    }

    [StorableConstructor]
    public Example(bool deserializing) { }

    [Storable]
    public string[] InputArgs { get; set; }
    [Storable]
    public string[] OutputArgs { get; set; }

    [Storable]
    public long[] InputInt { get; set; }
    [Storable]
    public double[] InputFloat { get; set; }
    [Storable]
    public bool[] InputBoolean { get; set; }
    [Storable]
    public char[] InputChar { get; set; }
    [Storable]
    public string[] InputString { get; set; }

    [Storable]
    public long[] OutputInt { get; set; }
    [Storable]
    public double[] OutputFloat { get; set; }
    [Storable]
    public bool[] OutputBoolean { get; set; }
    [Storable]
    public char[] OutputChar { get; set; }
    [Storable]
    public string[] OutputString { get; set; }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new Example(this, cloner);
    }
  }
}
