﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Erc {
  using System.Collections.Generic;
  using System.Linq;

  using HeuristicLab.Common;
  using HeuristicLab.Core;
  using HeuristicLab.Data;
  using HeuristicLab.Parameters;
  using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Erc.Interfaces;

  [StorableClass]
  public class StringErcOptions : ErcOption, IReadOnlyStringErcOptions {
    public const int DefaultStringLength = 10;

    private const string AllowLowercaseLettersParameterName = "Allow lowercase letters";
    private const string AllowUppercaseLettersParameterName = "Allow uppercase letters";
    private const string AllowSpaceParameterName = "Allow space";
    private const string SpaceProbabilityParameterName = "Space probability";
    private const string StringLengthParameterName = "String length";
    private const string ConstantsParameterName = "Constants";

    public StringErcOptions() {
      Parameters.Add(new FixedValueParameter<StringArray>(ConstantsParameterName));
      Parameters.Add(new FixedValueParameter<BoolValue>(AllowLowercaseLettersParameterName, new BoolValue(true)));
      Parameters.Add(new FixedValueParameter<BoolValue>(AllowUppercaseLettersParameterName, new BoolValue(true)));
      Parameters.Add(new FixedValueParameter<BoolValue>(AllowSpaceParameterName, new BoolValue(false)));
      Parameters.Add(new FixedValueParameter<IntValue>(StringLengthParameterName, new IntValue(DefaultStringLength)));
      Parameters.Add(new FixedValueParameter<PercentValue>(SpaceProbabilityParameterName));
    }

    [StorableConstructor]
    public StringErcOptions(bool deserializing) : base(deserializing) { }

    public StringErcOptions(StringErcOptions origin, Cloner cloner) : base(origin, cloner) { }

    public IValueParameter<BoolValue> AllowLowercaseLettersParameter
    {
      get { return (IValueParameter<BoolValue>)Parameters[AllowLowercaseLettersParameterName]; }
    }

    public bool AllowLowercaseLetters
    {
      get { return AllowLowercaseLettersParameter.Value.Value; }
      set { AllowLowercaseLettersParameter.Value.Value = value; }
    }

    public IValueParameter<BoolValue> AllowUppercaseLettersParameter
    {
      get { return (IValueParameter<BoolValue>)Parameters[AllowUppercaseLettersParameterName]; }
    }

    public bool AllowUppercaseLetters
    {
      get { return AllowUppercaseLettersParameter.Value.Value; }
      set { AllowUppercaseLettersParameter.Value.Value = value; }
    }

    public IValueParameter<BoolValue> AllowSpaceParameter
    {
      get { return (IValueParameter<BoolValue>)Parameters[AllowSpaceParameterName]; }
    }

    public bool AllowSpace
    {
      get { return AllowSpaceParameter.Value.Value; }
      set { AllowSpaceParameter.Value.Value = value; }
    }

    public IValueParameter<PercentValue> SpaceProbabilityParameter
    {
      get { return (IValueParameter<PercentValue>)Parameters[SpaceProbabilityParameterName]; }
    }

    public double SpaceProbability
    {
      get { return SpaceProbabilityParameter.Value.Value; }
      set { SpaceProbabilityParameter.Value.Value = value; }
    }

    public IValueParameter<IntValue> StringLengthParameter
    {
      get { return (IValueParameter<IntValue>)Parameters[StringLengthParameterName]; }
    }

    public int StringLength
    {
      get { return StringLengthParameter.Value.Value; }
      set { StringLengthParameter.Value.Value = value; }
    }

    public IValueParameter<StringArray> ConstantsParameter
    {
      get { return (IValueParameter<StringArray>)Parameters[ConstantsParameterName]; }
    }

    public string[] Constants
    {
      get { return ConstantsParameter.Value.ToArray(); }
      set { ConstantsParameter.Value = new StringArray(value); }
    }

    IReadOnlyList<string> IReadOnlyErcOptionConstants<string>.Constants
    {
      get { return Constants; }
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new StringErcOptions(this, cloner);
    }
  }
}
