﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using System.Collections.Generic;

  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;

  using Interpreter;
  using Stack;

  /// <summary>
  ///     Pushes the stack depth onto the INTEGER stack (thereby increasing it!).
  /// </summary>
  /// <typeparam name="T">Stacktype</typeparam>
  public abstract class StackdepthExpression<T> : StatelessExpression {
    public bool Eval(IPushStack<T> stack, IPushStack<long> integerStack, bool incremental = false) {
      var count = stack.Count;

      if (incremental) count += 1;

      integerStack.Push(count);
      return true;
    }
  }

  [PushExpression(StackTypes.Integer, "INTEGER.STACKDEPTH")]
  public class IntegerStackdepthExpression : StackdepthExpression<long> {

    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.IntegerStack, interpreter.IntegerStack, true);
    }
  }

  [PushExpression(StackTypes.Float, "FLOAT.STACKDEPTH", StackTypes.Integer)]
  public class FloatStackdepthExpression : StackdepthExpression<double> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.FloatStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackTypes.Boolean, "BOOLEAN.STACKDEPTH", StackTypes.Integer)]
  public class BooleanStackdepthExpression : StackdepthExpression<bool> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.BooleanStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackTypes.Name, "NAME.STACKDEPTH", StackTypes.Integer)]
  public class NameStackdepthExpression : StackdepthExpression<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.NameStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackTypes.Exec, "EXEC.STACKDEPTH", StackTypes.Integer)]
  public class ExecStackdepthExpression : StackdepthExpression<Expression> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.ExecStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackTypes.Code, "CODE.STACKDEPTH", StackTypes.Integer)]
  public class CodeStackdepthExpression : StackdepthExpression<Expression> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.CodeStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackTypes.Char, "CHAR.STACKDEPTH", StackTypes.Integer)]
  public class CharStackdepthExpression : StackdepthExpression<char> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.CharStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackTypes.String, "STRING.STACKDEPTH", StackTypes.Integer)]
  public class StringStackdepthExpression : StackdepthExpression<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.StringStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackTypes.IntegerVector, "INTEGER[].STACKDEPTH", StackTypes.Integer)]
  public class IntegerVectorStackdepthExpression : StackdepthExpression<List<long>> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.IntegerVectorStack, interpreter.IntegerStack);
    }
  }
}