﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using System;

  using HeuristicLab.Problems.ProgramSynthesis.Push.Data.Pool;

  [Serializable]
  public abstract class StatefulExpression<T> : Expression, IPooledObject {
    public T State { get; protected set; }
    private int? hashCode;
    private string stringRepresentation;

    protected StatefulExpression(T state) {
      this.State = state;
    }

    protected virtual int CalcHashCode() {
      return this.State.GetHashCode();
    }

    public override string StringRepresentation
    {
      get
      {
        if (stringRepresentation == null) stringRepresentation = this.State.ToString();
        return stringRepresentation;
      }
    }

    public override bool Equals(object obj) {
      if (ReferenceEquals(this, obj))
        return true;

      if (this.GetType() != obj.GetType())
        return false;

      var other = (StatefulExpression<T>)obj;

      return ReferenceEquals(this.State, other.State) ||
             this.GetHashCode() == other.GetHashCode();
    }

    public bool Equals(StatefulExpression<T> obj) {
      if (ReferenceEquals(this, obj))
        return true;

      if (GetType() != obj.GetType())
        return false;

      return ReferenceEquals(State, obj.State) ||
             GetHashCode() == obj.GetHashCode();
    }

    public override int GetHashCode() {
      if (hashCode == null) hashCode = CalcHashCode();
      return hashCode.Value;
    }

    public void Reset() {
      State = default(T);
    }
  }
}