﻿using System.Collections.Generic;

namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  /// <summary>
  /// Replaces all occurrences of the second lit-type item with the first lit-type item in the top type vector.
  /// </summary>
  /// <typeparam name="T"></typeparam>
  public abstract class VectorReplaceExpression<T> : StatelessExpression {
    protected bool Eval(IPushStack<List<T>> vectorStack, IPushStack<T> literalStack) {
      if (vectorStack.IsEmpty ||
          literalStack.Count < 2)
        return false;

      var vector = vectorStack.Top;
      var first = literalStack.Top;
      var second = literalStack.ReverseElementAt(1);
      literalStack.Remove(2);

      for (var i = 0; i < vector.Count; i++) {
        if (vector[i].Equals(second))
          vector[i] = first;
      }

      return true;
    }
  }

  [PushExpression(StackTypes.IntegerVector, "INTEGER[].REPLACE", StackTypes.Integer)]
  public class IntegerVectorReplaceExpression : VectorReplaceExpression<long> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.IntegerVectorStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackTypes.FloatVector, "FLOAT[].REPLACE", StackTypes.Float)]
  public class FloatVectorReplaceExpression : VectorReplaceExpression<double> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.FloatVectorStack, interpreter.FloatStack);
    }
  }

  [PushExpression(StackTypes.BooleanVector, "BOOLEAN[].REPLACE", StackTypes.Boolean)]
  public class BooleanVectorReplaceExpression : VectorReplaceExpression<bool> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.BooleanVectorStack, interpreter.BooleanStack);
    }
  }

  [PushExpression(StackTypes.StringVector, "STRING[].REPLACE", StackTypes.String)]
  public class StringVectorReplaceExpression : VectorReplaceExpression<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.StringVectorStack, interpreter.StringStack);
    }
  }
}
