﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Problem {

  using HeuristicLab.Core;
  using HeuristicLab.Encodings.IntegerVectorEncoding;
  using HeuristicLab.Optimization;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Configuration;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Data.Pool;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Expressions;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Generators.CodeGenerator;
  using HeuristicLab.Random;

  public static class IndividualMapper {
    public static PushProgram ToPushProgram(this Individual individual, IReadOnlyPushConfiguration config, ObjectPool<IRandom> randomPool) {
      return individual.IntegerVector().ToPushProgram(config, randomPool);
    }

    public static PushProgram ToPushProgram(this Individual individual, IReadOnlyPushConfiguration config) {
      return individual.IntegerVector().ToPushProgram(config);
    }

    public static PushProgram ToPushProgram(this IntegerVector vector, IReadOnlyPushConfiguration config, ObjectPool<IRandom> randomPool) {
      var random = randomPool.Allocate();
      var seed = vector.GetSeed();

      random.Reset(seed);
      var program = vector.ToPushProgram(config, random);
      randomPool.Free(random);

      return program;
    }

    public static PushProgram ToPushProgram(this IntegerVector vector, IReadOnlyPushConfiguration config) {
      var seed = (uint)vector.GetSeed();
      return vector.ToPushProgram(config, new MersenneTwister(seed));
    }

    private static int GetSeed(this IntegerVector vector) {
      var seed = 0;
      for (var i = 0; i < vector.Length; i++)
        seed += (i + 1) * vector[i];
      return seed;
    }

    private static PushProgram ToPushProgram(this IntegerVector vector, IReadOnlyPushConfiguration config, IRandom random) {
      var expressions = new Expression[vector.Length];

      for (var i = 0; i < vector.Length; i++) {
        expressions[i] = CodeGeneratorUtils.CreateExpressionOrErc(vector[i], random, config.EnabledExpressions, config.ErcOptions);
      }

      return new PushProgram(expressions);
    }
  }
}
