﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Problem {
  using HeuristicLab.BenchmarkSuite.Problems;
  using HeuristicLab.Common;
  using HeuristicLab.Core;
  using HeuristicLab.Encodings.IntegerVectorEncoding;
  using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Configuration;

  [StorableClass]
  public class PushSolution : NamedItem {
    [Storable]
    public readonly double Quality;
    [Storable]
    public readonly IntegerVector IntegerVector;
    [Storable]
    public readonly Data Data;
    [Storable]
    public readonly IRandom Random;
    [Storable]
    public readonly IReadOnlyPushConfiguration Config;
    [Storable]
    public readonly int DataStart;
    [Storable]
    public readonly int DataEnd;

    [Storable]
    public readonly bool Simplify;

    public PushSolution(IntegerVector integerVector, double quality, Data data, IRandom random, IReadOnlyPushConfiguration config, int dataStart, int dataEnd, bool simplify = false)
      : base("Solution", "A push solution.") {
      IntegerVector = integerVector;
      Quality = quality;
      Data = data;
      Random = random;
      Config = config;
      DataStart = dataStart;
      DataEnd = dataEnd;
      Simplify = simplify;
    }

    public PushSolution(PushSolution origin, Cloner cloner) : base(origin, cloner) {
      IntegerVector = cloner.Clone(origin.IntegerVector);
      Quality = origin.Quality;
      Data = cloner.Clone(origin.Data);
      Random = cloner.Clone(origin.Random);
      Config = cloner.Clone(origin.Config);
      DataStart = origin.DataStart;
      DataEnd = origin.DataEnd;
      Simplify = origin.Simplify;
    }

    [StorableConstructor]
    private PushSolution(bool deserializing) : base(deserializing) { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new PushSolution(this, cloner);
    }
  }
}
